@@ -26,9 +26,12 @@
 
 """Read KLM data.
 
-Reads L1b GAC/LAC data from KLM series of satellites (NOAA-15 and later) and does most of the computations.
-Format specification can be found here:
-http://www.ncdc.noaa.gov/oa/pod-guide/ncdc/docs/klm/html/c8/sec83142-1.htm
+Reads L1b GAC/LAC data from KLM series of satellites (NOAA-15 and later).
+Format specification can be found in section 8 of the `KLM user guide`_.
+
+.. _KLM user guide:
+    https://www.ncei.noaa.gov/pub/data/satellite/publications/podguides/N-15%20thru%20N-19/
+
 """
 
 import datetime
@@ -51,48 +54,53 @@ LOG = logging.getLogger(__name__)
 class KLM_QualityIndicator(IntFlag):
     """Quality Indicators.
 
-    Source:
-        KLM guide
-        Table 8.3.1.3.3.1-1. Format of packed LAC/HRPT Data Sets (Version 2, pre-April 28, 2005).
-        Table 8.3.1.3.3.2-1. Format of LAC/HRPT Data Record for NOAA-N (Version 5, post-November 14,
-                             2006, all spacecraft).
-        Table 8.3.1.4.3.1-1. Format of packed GAC Data Record for NOAA KLM (Version 2, pre-April 28, 2005).
-        Table 8.3.1.4.3.2-1. Format of GAC Data Record for NOAA-N (Version 4, post-January 25, 2006,
-                             all spacecraft).
+    Source: KLM guide
+
+    - Table 8.3.1.3.3.1-1. Format of packed LAC/HRPT Data Sets (Version 2,
+      pre-April 28, 2005).
+    - Table 8.3.1.3.3.2-1. Format of LAC/HRPT Data Record for NOAA-N
+      (Version 5, post-November 14, 2006, all spacecraft).
+    - Table 8.3.1.4.3.1-1. Format of packed GAC Data Record for NOAA KLM
+      (Version 2, pre-April 28, 2005).
+    - Table 8.3.1.4.3.2-1. Format of GAC Data Record for NOAA-N (Version 4,
+      post-January 25, 2006, all spacecraft).
+
+    Notes:
+
+    - Table 8.3.1.3.3.1-1. and Table 8.3.1.4.3.1-1. define bit: 21 as
+      "frame sync word not valid"
+    - Table 8.3.1.3.3.2-1. and Table 8.3.1.4.3.2-1. define bit: 21 as
+      "flywheeling detected during this frame"
 
-    Note:
-        Table 8.3.1.3.3.1-1. and Table 8.3.1.4.3.1-1. define bit: 21 as
-        "frame sync word not valid"
-        Table 8.3.1.3.3.2-1. and Table 8.3.1.4.3.2-1. define bit: 21 as
-        "flywheeling detected during this frame"
     """
-    FATAL_FLAG = 2**0  # Data should not be used for product generation
-    TIME_ERROR = 2**1  # Time sequence error detected within this scan
-    DATA_GAP = 2**2  # Data gap precedes this scan
-    CALIBRATION = 2**3  # Insufficient data for calibration
-    NO_EARTH_LOCATION = 2**4  # Earth location data not available
-    CLOCK_UPDATE = 2**5  # First good time following a clock update (nominally 0)
-    INSTRUMENT_CHANGE = 2**6  # Instrument status changed with this scan
-    BIT_SYNC_STATUS = 2**7  # Sync lock dropped during this frame
-    SYNC_ERROR = 2**8  # Frame sync word error greater than zero
-    FRAME_SYNC_LOCK = 2**9  # Frame sync previously dropped lock
-    SYNC_INVALID = 2**10  # Frame sync word not valid
-    FLYWHEELING = 2**10  # Flywheeling detected during this frame
-    BIT_SLIPPAGE = 2**11  # Bit slippage detected during this frame
-    # Note: 2**12 to 2**22 are not defined for KLMs
-    TIP_PARITY = 2**23  # TIP parity error detected
+
+    FATAL_FLAG = 2**31  # Data should not be used for product generation
+    TIME_ERROR = 2**30  # Time sequence error detected within this scan
+    DATA_GAP = 2**29  # Data gap precedes this scan
+    CALIBRATION = 2**28  # Insufficient data for calibration
+    NO_EARTH_LOCATION = 2**27  # Earth location data not available
+    CLOCK_UPDATE = 2**26  # First good time following a clock update (nominally 0)
+    INSTRUMENT_CHANGE = 2**25  # Instrument status changed with this scan
+    BIT_SYNC_STATUS = 2**24  # Sync lock dropped during this frame
+    SYNC_ERROR = 2**23  # Frame sync word error greater than zero
+    FRAME_SYNC_LOCK = 2**22  # Frame sync previously dropped lock
+    SYNC_INVALID = 2**21  # Frame sync word not valid
+    FLYWHEELING = 2**21  # Flywheeling detected during this frame
+    BIT_SLIPPAGE = 2**20  # Bit slippage detected during this frame
+    # Note: Bit 19 - 9 are not defined for KLMs
+    TIP_PARITY = 2**8  # TIP parity error detected
     # Reflected Sunlight (RS) detected (solar blackbody contamination)
-    CH_3B_RS = 2**24
-    CH_3_CONTAMINATION = 2**24  # POD compatible alias
-    CH_3B_RS_ANOMALY = 2**25
-    CH_4_RS = 2**26
-    CH_4_CONTAMINATION = 2**26  # POD compatible alias
-    CH_4_RS_ANOMALY = 2**27
-    CH_5_RS = 2**28
-    CH_5_CONTAMINATION = 2**28  # POD compatible alias
-    CH_5_RS_ANOMALY = 2**29
-    DATA_JITTER = 2**30  # Resync occurred on this frame
-    PSEUDO_NOISE = 2**31  # Pseudo noise occurred on this frame
+    CH_3B_RS = 2**7
+    CH_3B_RS_ANOMALY = 2**6
+    CH_3_CONTAMINATION = CH_3B_RS | CH_3B_RS_ANOMALY  # POD compatible alias
+    CH_4_RS = 2**5
+    CH_4_RS_ANOMALY = 2**4
+    CH_4_CONTAMINATION = CH_4_RS | CH_4_RS_ANOMALY  # POD compatible alias
+    CH_5_RS = 2**3
+    CH_5_RS_ANOMALY = 2**2
+    CH_5_CONTAMINATION = CH_5_RS | CH_5_RS_ANOMALY  # POD compatible alias
+    DATA_JITTER = 2**1  # Resync occurred on this frame
+    PSEUDO_NOISE = 2**0  # Pseudo noise occurred on this frame
 
 
 # GAC header object
@@ -616,6 +624,7 @@ class KLMReader(Reader):
                         8: 'noaa19',
                         12: 'metopa',
                         11: 'metopb',
+                        13: 'metopc',
                         }
     spacecrafts_orbital = {4: 'noaa 15',
                            2: 'noaa 16',
@@ -624,6 +633,7 @@ class KLMReader(Reader):
                            8: 'noaa 19',
                            12: 'metop 02',
                            11: 'metop 01',
+                           13: 'metop 03',
                            }
 
     tsm_affected_intervals = TSM_AFFECTED_INTERVALS_KLM
@@ -803,8 +813,11 @@ class KLMReader(Reader):
         channels[:, :, 3][switch == 2] = np.nan
 
     def _adjust_clock_drift(self):
-        """Clock drift correction is only applied to POD satellites."""
-        pass
+        """Adjust the geolocation to compensate for the clock error.
+        Note:
+            Clock drift correction is only applied to POD satellites.
+            On the KLM series, the clock is updated daily.
+        """
 
     def get_tsm_pixels(self, channels):
         """Determine pixels affected by the scan motor issue.
