@@ -33,6 +33,7 @@ import re
 import six
 import types
 import warnings
+import pyorbital
 
 from pygac.utils import (centered_modulus,
                          calculate_sun_earth_distance_correction,
@@ -41,6 +42,7 @@ from pyorbital.orbital import Orbital
 from pyorbital import astronomy
 from pygac.calibration import Calibrator, calibrate_solar, calibrate_thermal
 from pygac import gac_io
+from distutils.version import LooseVersion
 
 LOG = logging.getLogger(__name__)
 
@@ -78,8 +80,12 @@ class ReaderError(ValueError):
     pass
 
 
+class NoTLEData(IndexError):
+    """Raised if no TLE data available within time range."""
+
+
 class Reader(six.with_metaclass(ABCMeta)):
-    """Reader for Gac and Lac, POD and KLM data."""
+    """Reader for GAC and LAC format, POD and KLM platforms."""
 
     # data set header format, see _validate_header for more details
     data_set_pattern = re.compile(
@@ -121,10 +127,15 @@ class Reader(six.with_metaclass(ABCMeta)):
         self.utcs = None
         self.lats = None
         self.lons = None
-        self.times = None
         self.tle_lines = None
         self.filename = None
         self._mask = None
+        self._rpy = None
+
+    @property
+    def times(self):
+        """The UTCs as datetime.datetime"""
+        return self.to_datetime(self.utcs)
 
     @property
     def filename(self):
@@ -137,6 +148,7 @@ class Reader(six.with_metaclass(ABCMeta)):
         if filepath is None:
             self._filename = None
         else:
+            filepath = os.fspath(filepath)
             match = self.data_set_pattern.search(filepath)
             if match:
                 self._filename = match.group()
@@ -154,7 +166,7 @@ class Reader(six.with_metaclass(ABCMeta)):
         return calibration
 
     @abstractmethod
-    def read(self, filename, fileobj=None):
+    def read(self, filename, fileobj=None):  # pragma: no cover
         """Read the GAC/LAC data.
 
         Args:
@@ -165,7 +177,7 @@ class Reader(six.with_metaclass(ABCMeta)):
 
     @classmethod
     @abstractmethod
-    def read_header(cls, filename, fileobj=None):
+    def read_header(cls, filename, fileobj=None):  # pragma: no cover
         """Read the file header.
 
         Args:
@@ -190,6 +202,7 @@ class Reader(six.with_metaclass(ABCMeta)):
             header (struct): file header
             filename (str): path to file
         """
+        filename = str(filename)
         data_set_name = header['data_set_name'].decode(errors='ignore')
         if not cls.data_set_pattern.match(data_set_name):
             LOG.debug('The data_set_name in header %s does not match.'
@@ -341,7 +354,7 @@ class Reader(six.with_metaclass(ABCMeta)):
         )
 
     @abstractmethod
-    def get_header_timestamp(self):
+    def get_header_timestamp(self):  # pragma: no cover
         """Read start timestamp from the header.
 
         Returns:
@@ -387,7 +400,7 @@ class Reader(six.with_metaclass(ABCMeta)):
             return channels
 
     @abstractmethod
-    def _get_times(self):
+    def _get_times(self):  # pragma: no cover
         """Specify how to read scanline timestamps from GAC data.
 
         Returns:
@@ -418,9 +431,6 @@ class Reader(six.with_metaclass(ABCMeta)):
             self.utcs = self.to_datetime64(year=year, jday=jday, msec=msec)
             self.correct_times_thresh()
 
-            # Convert timestamps to datetime objects
-            self.times = self.to_datetime(self.utcs)
-
         return self.utcs
 
     @staticmethod
@@ -436,8 +446,8 @@ class Reader(six.with_metaclass(ABCMeta)):
             numpy.datetime64: Converted timestamps
 
         """
-        return (((year - 1970).astype('datetime64[Y]')
-                 + (jday - 1).astype('timedelta64[D]')).astype('datetime64[ms]')
+        return (year.astype(str).astype('datetime64[Y]')
+                + (jday - 1).astype('timedelta64[D]')
                 + msec.astype('timedelta64[ms]'))
 
     @staticmethod
@@ -468,74 +478,6 @@ class Reader(six.with_metaclass(ABCMeta)):
         """
         return (scan_line_number - 1) / self.scan_freq
 
-    def compute_lonlat(self, width, utcs=None, clock_drift_adjust=True):
-        """Compute lat/lon coordinates.
-
-        Args:
-            width: Number of coordinates per scanlines
-            utcs: Scanline timestamps
-            clock_drift_adjust: If True, adjust clock drift.
-
-        """
-        if utcs is None:
-            utcs = self.get_times()
-
-        # adjusting clock for drift
-        tic = datetime.datetime.now()
-        if clock_drift_adjust:
-            from pygac.clock_offsets_converter import get_offsets
-            try:
-                offset_times, clock_error = get_offsets(self.spacecraft_name)
-            except KeyError:
-                LOG.info("No clock drift info available for %s",
-                         self.spacecraft_name)
-            else:
-                offset_times = np.array(offset_times, dtype='datetime64[ms]')
-                offsets = np.interp(utcs.astype(np.uint64),
-                                    offset_times.astype(np.uint64),
-                                    clock_error)
-                utcs = utcs - (offsets * 1000).astype('timedelta64[ms]')
-
-        t = utcs[0].astype(datetime.datetime)
-
-        if "constant_yaw_attitude_error" in self.head.dtype.fields:
-            rpy = np.deg2rad([self.head["constant_roll_attitude_error"] / 1e3,
-                              self.head["constant_pitch_attitude_error"] / 1e3,
-                              self.head["constant_yaw_attitude_error"] / 1e3])
-        else:
-            try:
-                # This needs to be checked thoroughly first
-                # rpy_spacecraft = rpy_coeffs[self.spacecraft_name]
-                # rpy = [rpy_spacecraft['roll'],
-                #        rpy_spacecraft['pitch'],
-                #        rpy_spacecraft['yaw']]
-                # LOG.debug("Using static attitude correction")
-                raise KeyError
-            except KeyError:
-                LOG.debug("Not applying attitude correction")
-                rpy = [0, 0, 0]
-
-        LOG.info("Using rpy: %s", str(rpy))
-
-        from pyorbital.geoloc_instrument_definitions import avhrr_gac
-        from pyorbital.geoloc import compute_pixels, get_lonlatalt
-        # TODO: Are we sure all satellites have this scan width in degrees ?
-        sgeom = avhrr_gac(utcs.astype(datetime.datetime),
-                          self.scan_points, 55.385)
-        s_times = sgeom.times(t)
-        tle1, tle2 = self.get_tle_lines()
-
-        pixels_pos = compute_pixels((tle1, tle2), sgeom, s_times, rpy)
-        pos_time = get_lonlatalt(pixels_pos, s_times)
-
-        toc = datetime.datetime.now()
-
-        LOG.warning("Computation of geolocation: %s", str(toc - tic))
-
-        lons, lats = pos_time[:2]
-
-        return lons.reshape(-1, width), lats.reshape(-1, width)
-
     def get_sun_earth_distance_correction(self):
         """Get the julian day and the sun-earth distance correction."""
         self.get_times()
@@ -559,9 +501,10 @@ class Reader(six.with_metaclass(ABCMeta)):
         """Calibrate and return the channels."""
         channels = self.get_counts()
         self.get_times()
+        times = self.times
         self.update_meta_data()
-        year = self.times[0].year
-        delta = self.times[0].date() - datetime.date(year, 1, 1)
+        year = times[0].year
+        delta = times[0].date() - datetime.date(year, 1, 1)
         jday = delta.days + 1
 
         corr = self.meta_data['sun_earth_distance_correction_factor']
@@ -602,6 +545,11 @@ class Reader(six.with_metaclass(ABCMeta)):
 
         return channels
 
+    @abstractmethod
+    def get_telemetry(self):  # pragma: no cover
+        """KLM/POD specific readout of telemetry."""
+        raise NotImplementedError
+
     def get_lonlat(self):
         """Compute lat/lon coordinates.
 
@@ -631,7 +579,7 @@ class Reader(six.with_metaclass(ABCMeta)):
         return self.lons, self.lats
 
     @abstractmethod
-    def _get_lonlat(self):
+    def _get_lonlat(self):  # pragma: no cover
         """KLM/POD specific readout of lat/lon coordinates."""
         raise NotImplementedError
 
@@ -642,6 +590,15 @@ class Reader(six.with_metaclass(ABCMeta)):
             self._mask = self._get_corrupt_mask()
         return self._mask
 
+    @abstractproperty
+    def QFlag(self):  # pragma: no cover
+        """KLM/POD specific quality indicators."""
+        raise NotImplementedError
+
+    @abstractproperty
+    def _quality_indicators_key(self):  # pragma: no cover
+        raise NotImplementedError
+
     def _get_corrupt_mask(self, flags=None):
         """Readout of corrupt scanline mask.
 
@@ -672,12 +629,12 @@ class Reader(six.with_metaclass(ABCMeta)):
         return qual_flags
 
     @abstractmethod
-    def postproc(self, channels):
+    def postproc(self, channels):  # pragma: no cover
         """Apply KLM/POD specific postprocessing."""
         raise NotImplementedError
 
     @abstractmethod
-    def _adjust_clock_drift(self):
+    def _adjust_clock_drift(self):  # pragma: no cover
         """Adjust clock drift."""
         raise NotImplementedError
 
@@ -731,7 +688,7 @@ class Reader(six.with_metaclass(ABCMeta)):
             return self.tle_lines
         self.get_times()
         tle_data = self.read_tle_file(self.get_tle_file())
-        sdate = np.datetime64(self.times[0], '[ms]')
+        sdate = self.utcs[0]
         dates = self.tle2datetime64(
             np.array([float(line[18:32]) for line in tle_data[::2]]))
 
@@ -752,7 +709,7 @@ class Reader(six.with_metaclass(ABCMeta)):
         # Make sure the TLE we found is within the threshold
         delta_days = abs(sdate - dates[iindex]) / np.timedelta64(1, 'D')
         if delta_days > self.tle_thresh:
-            raise IndexError(
+            raise NoTLEData(
                 "Can't find tle data for %s within +/- %d days around %s" %
                 (self.spacecraft_name, self.tle_thresh, sdate))
 
@@ -769,6 +726,47 @@ class Reader(six.with_metaclass(ABCMeta)):
         self.tle_lines = tle1, tle2
         return tle1, tle2
 
+    def get_sat_angles(self):
+        """Get satellite angles.
+
+        Returns:
+            Azimuth, elevation (degrees)
+        """
+        try:
+            return self._get_sat_angles_with_tle()
+        except NoTLEData:
+            LOG.warning(
+                'No TLE data available. Falling back to approximate '
+                'calculation of satellite angles.'
+            )
+            return self._get_sat_angles_without_tle()
+
+    def _get_sat_angles_with_tle(self):
+        tle1, tle2 = self.get_tle_lines()
+        orb = Orbital(self.spacecrafts_orbital[self.spacecraft_id],
+                      line1=tle1, line2=tle2)
+        sat_azi, sat_elev = orb.get_observer_look(self.times[:, np.newaxis],
+                                                  self.lons, self.lats, 0)
+        return sat_azi, sat_elev
+
+    def _get_sat_angles_without_tle(self):
+        """Get satellite angles using lat/lon from data to approximate satellite postition instead of TLE."""
+        from pyorbital.orbital import get_observer_look as get_observer_look_no_tle
+        LOG.warning('Approximating satellite height to 850km (TIROS-N OSCAR)!')
+        sat_alt = 850.0  # km  TIROS-N OSCAR
+        mid_column = int(0.5*self.lons.shape[1])
+        sat_azi, sat_elev = get_observer_look_no_tle(
+            self.lons[:, mid_column][:, np.newaxis],
+            self.lats[:, mid_column][:, np.newaxis],  # approximate satellite position
+            sat_alt,  # approximate satellite altitude
+            self.times[:, np.newaxis],
+            self.lons, self.lats, 0)
+        # Sometimes (pyorbital <= 1.6.1) the get_observer_look_not_tle returns nodata instead of 90.
+        # Problem solved with https://github.com/pytroll/pyorbital/pull/77
+        if LooseVersion(pyorbital.__version__) <= LooseVersion('1.6.1'):
+            sat_elev[:, mid_column] = 90
+        return sat_azi, sat_elev
+
     def get_angles(self):
         """Get azimuth and zenith angles.
 
@@ -777,31 +775,31 @@ class Reader(six.with_metaclass(ABCMeta)):
         and different ranges.
 
         Returns:
-            sat_azi: satellite azimuth angle
-                degree clockwise from north in range ]-180, 180],
-            sat_zentih: satellite zenith angles in degrees in range [0,90],
-            sun_azi: sun azimuth angle
-                degree clockwise from north in range ]-180, 180],
-            sun_zentih: sun zenith angles in degrees in range [0,90],
+
+            sat_azi: satellite azimuth angle degree clockwise from north in
+            range ]-180, 180]
+
+            sat_zentih: satellite zenith angles in degrees in range [0,90]
+
+            sun_azi: sun azimuth angle degree clockwise from north in range
+            ]-180, 180]
+
+            sun_zentih: sun zenith angles in degrees in range [0,90]
+
             rel_azi: absolute azimuth angle difference in degrees between sun
-                and sensor in range [0, 180]
+            and sensor in range [0, 180]
 
         """
         self.get_times()
         self.get_lonlat()
-        tle1, tle2 = self.get_tle_lines()
-        orb = Orbital(self.spacecrafts_orbital[self.spacecraft_id],
-                      line1=tle1, line2=tle2)
-
-        sat_azi, sat_elev = orb.get_observer_look(self.times[:, np.newaxis],
-                                                  self.lons, self.lats, 0)
+        times = self.times
+        sat_azi, sat_elev = self.get_sat_angles()
 
         sat_zenith = 90 - sat_elev
-
-        sun_zenith = astronomy.sun_zenith_angle(self.times[:, np.newaxis],
+        sun_zenith = astronomy.sun_zenith_angle(times[:, np.newaxis],
                                                 self.lons, self.lats)
 
-        alt, sun_azi = astronomy.get_alt_az(self.times[:, np.newaxis],
+        alt, sun_azi = astronomy.get_alt_az(times[:, np.newaxis],
                                             self.lons, self.lats)
         del alt
         sun_azi = np.rad2deg(sun_azi)
@@ -872,7 +870,7 @@ class Reader(six.with_metaclass(ABCMeta)):
                 msec0 = np.median(msec - msec_lineno)
                 msec = msec0 + msec_lineno
 
-        return year, jday, msec
+        return year.astype(int), jday.astype(int), msec
 
     def correct_scan_line_numbers(self):
         """Remove scanlines with corrupted scanline numbers.
@@ -880,7 +878,7 @@ class Reader(six.with_metaclass(ABCMeta)):
         This includes:
             - Scanline numbers outside the valide range
             - Scanline numbers deviating more than a certain threshold from the
-            ideal case (1,2,3,...N)
+              ideal case (1,2,3,...N)
 
         Example files having corrupt scanline numbers:
             - NSS.GHRR.NJ.D96144.S2000.E2148.B0720102.GC
@@ -1052,7 +1050,7 @@ class Reader(six.with_metaclass(ABCMeta)):
         return results
 
     @abstractproperty
-    def tsm_affected_intervals(self):
+    def tsm_affected_intervals(self):  # pragma: no cover
         """Specify time intervals being affected by the scan motor problem.
 
         Returns:
@@ -1071,8 +1069,7 @@ class Reader(six.with_metaclass(ABCMeta)):
 
         """
         self.get_times()
-        ts = self.times[0]
-        te = self.times[-1]
+        ts, te = self.to_datetime(self.utcs[[0, -1]])
         try:
             for interval in self.tsm_affected_intervals[self.spacecraft_id]:
                 if ts >= interval[0] and te <= interval[1]:
@@ -1126,13 +1123,36 @@ class Reader(six.with_metaclass(ABCMeta)):
         channels[idx] = np.nan
 
     @abstractmethod
-    def get_tsm_pixels(self, channels):
+    def get_tsm_pixels(self, channels):  # pragma: no cover
         """Determine pixels affected by the scan motor issue.
 
         Channel selection is POD/KLM specific.
         """
         raise NotImplementedError
 
+    def get_attitude_coeffs(self):
+        """Return the roll, pitch, yaw values"""
+        if self._rpy is None:
+            if "constant_yaw_attitude_error" in self.head.dtype.fields:
+                rpy = np.deg2rad([self.head["constant_roll_attitude_error"] / 1e3,
+                                  self.head["constant_pitch_attitude_error"] / 1e3,
+                                  self.head["constant_yaw_attitude_error"] / 1e3])
+            else:
+                try:
+                    # This needs to be checked thoroughly first
+                    # rpy_spacecraft = rpy_coeffs[self.spacecraft_name]
+                    # rpy = np.array([rpy_spacecraft['roll'],
+                    #                 rpy_spacecraft['pitch'],
+                    #                 rpy_spacecraft['yaw']])
+                    # LOG.debug("Using static attitude correction")
+                    raise KeyError
+                except KeyError:
+                    LOG.debug("Not applying attitude correction")
+                    rpy = np.zeros(3)
+            LOG.info("Using rpy: %s", str(rpy))
+            self._rpy = rpy
+        return self._rpy
+
 
 def inherit_doc(cls):
     """Make a class method inherit its docstring from the parent class.
