@@ -23,6 +23,7 @@
 import datetime
 import unittest
 import sys
+import os
 try:
     import mock
 except ImportError:
@@ -30,6 +31,69 @@ except ImportError:
 import numpy as np
 import numpy.testing
 from pygac.gac_reader import GACReader, ReaderError
+from pygac.pod_reader import POD_QualityIndicator
+from pygac.gac_pod import scanline
+from pygac.reader import NoTLEData
+
+
+class TestPath(os.PathLike):
+    def __init__(self, path):
+        self.path = str(path)
+
+    def __fspath__(self):
+        return self.path
+
+
+class FakeGACReader(GACReader):
+    QFlag = POD_QualityIndicator
+    _quality_indicators_key = "quality_indicators"
+    tsm_affected_intervals = {None: []}
+    along_track = 3
+    across_track = 4
+
+    def __init__(self):
+        super(FakeGACReader, self).__init__()
+        self.scan_width = self.across_track
+        scans = np.zeros(self.along_track, dtype=scanline)
+        scans["scan_line_number"] = np.arange(self.along_track)
+        scans["sensor_data"] = 128
+        self.scans = scans
+        self.head = {'foo': 'bar'}
+        self.spacecraft_name = 'noaa6'
+
+    def _get_times(self):
+        year = np.full(self.along_track, 1970, dtype=int)
+        jday = np.full(self.along_track, 1, dtype=int)
+        msec = 1000 * np.arange(1, self.along_track+1, dtype=int)
+        return year, jday, msec
+
+    def get_header_timestamp(self):
+        return datetime.datetime(1970, 1, 1)
+
+    def get_telemetry(self):
+        prt = 51 * np.ones(self.along_track)  # prt threshold is 50
+        ict = 101 * np.ones((self.along_track, 3))  # ict threshold is 100
+        space = 101 * np.ones((self.along_track, 3))  # space threshold is 100
+        return prt, ict, space
+
+    def _adjust_clock_drift(self):
+        pass
+
+    def _get_lonlat(self):
+        pass
+
+    def postproc(self, channels):
+        pass
+
+    def read(self, filename, fileobj=None):
+        pass
+
+    @classmethod
+    def read_header(cls, filename, fileobj=None):
+        pass
+
+    def get_tsm_pixels(self, channels):
+        pass
 
 
 class TestGacReader(unittest.TestCase):
@@ -55,12 +119,16 @@ class TestGacReader(unittest.TestCase):
         filepath = '/path/to/' + filename + '.gz'
         self.reader.filename = filepath
         self.assertEqual(self.reader.filename, filename)
+        self.reader.filename = None
+        self.assertIsNone(self.reader.filename)
+        self.reader.filename = TestPath(filepath)
+        self.assertEqual(self.reader.filename, filename)
 
     @unittest.skipIf(sys.version_info.major < 3, "Skipped in python2!")
     def test__read_scanlines(self):
         """Test the scanline extraction."""
         self.reader.scanline_type = np.dtype([
-            ('a', 'S2'), ('b', 'i4')])
+            ('a', 'S2'), ('b', '<i4')])
         # request more scan lines than available
         with self.assertWarnsRegex(RuntimeWarning,
                                    "Unexpected number of scanlines!"):
@@ -109,6 +177,10 @@ class TestGacReader(unittest.TestCase):
         # should be fine, because the data_set_name is the pefered source
         head = self.reader._correct_data_set_name(val_head.copy(), inv_filepath)
         self.assertEqual(head['data_set_name'], val_head['data_set_name'])
+        # enter a valid data_set_name, and an FSFile/pathlib object as filepath
+        fs_filepath = TestPath(val_filepath)
+        head = self.reader._correct_data_set_name(val_head.copy(), fs_filepath)
+        self.assertEqual(head['data_set_name'], val_filename.encode())
 
     @mock.patch('pygac.reader.Reader.get_calibrated_channels')
     def test__get_calibrated_channels_uniform_shape(self, get_channels):
@@ -119,6 +191,21 @@ class TestGacReader(unittest.TestCase):
         with self.assertRaises(AssertionError):
             self.reader._get_calibrated_channels_uniform_shape()
 
+    def test_get_calibrated_channels(self):
+        reader = FakeGACReader()
+        res = reader.get_calibrated_channels()
+        expected = np.full(
+            (
+                FakeGACReader.along_track,
+                FakeGACReader.across_track,
+                5  # number of channels
+            ),
+            np.nan
+        )
+        expected[:, 1, 0] = 8.84714652
+        expected[:, 2, 1] = 10.23511303
+        np.testing.assert_allclose(res, expected)
+
     def test_to_datetime64(self):
         """Test conversion from (year, jday, msec) to datetime64."""
         t0 = GACReader.to_datetime64(year=np.array(1970), jday=np.array(1),
@@ -240,7 +327,6 @@ class TestGacReader(unittest.TestCase):
 
         for utcs, scanline in zip((utcs1, utcs2), (scanline1, scanline2)):
             self.reader.utcs = utcs
-            self.reader.times = utcs.astype(datetime.datetime)
             self.assertEqual(self.reader.get_midnight_scanline(), scanline,
                              msg='Incorrect midnight scanline')
 
@@ -289,15 +375,82 @@ class TestGacReader(unittest.TestCase):
 
         read_tle_file.return_value = tle_data
         for time, tle_idx in expected.items():
-            self.reader.times = [time]
+            self.reader.utcs = np.array([time], dtype='datetime64[ms]')
             self.reader.tle_lines = None
             if tle_idx is None:
-                self.assertRaises(IndexError, self.reader.get_tle_lines)
+                self.assertRaises(NoTLEData, self.reader.get_tle_lines)
             else:
                 tle1, tle2 = self.reader.get_tle_lines()
                 self.assertEqual(tle1, tle_data[tle_idx])
                 self.assertEqual(tle2, tle_data[tle_idx + 1])
 
+    @mock.patch('pygac.reader.Reader.get_tle_lines')
+    def test_get_sat_angles_without_tle_at_nadir(self, get_tle_lines):
+        """Test that the get satellite angles without tle at nadir."""
+        get_tle_lines.side_effect = NoTLEData('No TLE data available')
+        rng = np.random.RandomState(125)
+        self.reader.lons = rng.rand(100, 409) * 90
+        self.reader.lats = rng.rand(100, 409) * 90
+        self.reader.utcs = np.array(
+            [numpy.datetime64(datetime.datetime(1980, 1, 3, 11, 47, 15, 469000)) for date in range(100)])
+        sat_azi, sat_elev = self.reader.get_sat_angles()
+        self.assertEqual(np.sum(np.isnan(sat_elev)), 0)
+        np.testing.assert_allclose(sat_elev[:, 204], 90., atol=0.01)
+
+    @mock.patch('pygac.reader.Reader.get_tle_lines')
+    def test_get_sat_angles_without_tle(self, get_tle_lines):
+        """Test the get satellite angles without tle."""
+        get_tle_lines.side_effect = NoTLEData('No TLE data available')
+
+        # Test data correspond to columns 0:2, 201:208 and 407:409. Extracted like this:
+        # self.lons[0:5, [0, 1, 201, 202, 203, 204, 205, 206, 207, -2, -1]]
+        self.reader.lons = np.array([[69.41555135, 68.76815744, 28.04133742, 27.94671757, 27.85220562,
+                                      27.7578125, 27.66354783, 27.5694182, 27.47542957,  2.66416611,
+                                      2.39739436],
+                                     [69.41409536, 68.76979616, 28.00228658, 27.9076628, 27.8131467,
+                                      27.71875, 27.62448295, 27.53035209, 27.43636312,  2.61727408,
+                                      2.35049275],
+                                     [69.42987929, 68.78543423, 27.96407251, 27.86936406, 27.77457923,
+                                      27.6796875, 27.58467527, 27.48959853, 27.39453053,  2.5704025,
+                                      2.30362323],
+                                     [69.44430772, 68.80064104, 27.91910034, 27.82340242, 27.72794715,
+                                      27.6328125, 27.53805662, 27.44366144, 27.34959008,  2.53088093,
+                                      2.26729486],
+                                     [69.47408815, 68.8259859, 27.87666513, 27.78224611, 27.68795045,
+                                      27.59375, 27.49962326, 27.40557682, 27.31162435,  2.48359319,
+                                      2.21976689]])
+        self.reader.lats = np.array([[71.62830288, 71.67081539, 69.90976034, 69.89297223, 69.87616536,
+                                      69.859375, 69.84262997, 69.82593315, 69.80928089, 61.61334632,
+                                      61.466097],
+                                     [71.65222644, 71.69455292, 69.9331981, 69.91640991, 69.89960295,
+                                      69.8828125, 69.86606741, 69.84937054, 69.83271828, 61.62903602,
+                                      61.48182119],
+                                     [71.68324681, 71.7256586, 69.96444365, 69.94765637, 69.93085095,
+                                      69.9140625, 69.89731955, 69.8806245, 69.86397337, 61.65247459,
+                                      61.50526043],
+                                     [71.71489701, 71.75715746, 69.98789144, 69.97110185, 69.9542928,
+                                      69.9375, 69.92075278, 69.90405431, 69.88740114, 61.66489162,
+                                      61.51597801],
+                                     [71.7390608, 71.78108857, 70.01126173, 69.99449392, 69.97770882,
+                                      69.9609375, 69.9442054, 69.92751555, 69.91086544, 61.67769951,
+                                      61.52802445]])
+        self.reader.utcs = np.array([numpy.datetime64(datetime.datetime(1980, 1, 3, 11, 47, 15, 469000)),
+                                     numpy.datetime64(datetime.datetime(1980, 1, 3, 11, 47, 15, 969000)),
+                                     numpy.datetime64(datetime.datetime(1980, 1, 3, 11, 47, 16, 469000)),
+                                     numpy.datetime64(datetime.datetime(1980, 1, 3, 11, 47, 16, 969000)),
+                                     numpy.datetime64(datetime.datetime(1980, 1, 3, 11, 47, 17, 469000))])
+        expected_sat_azi_0 = np.array([283.09872924, 283.12775589, 283.13951497, 283.14786413, 283.19638805])
+        expected_sat_azi_201 = np.array([272.85051989, 273.79847634, 272.04794616, 273.01363377, 274.00055397])
+        expected_sat_azi_408 = np.array([39.77021472, 39.71516966, 39.68104134, 39.60503726, 39.5403431])
+        expected_sat_elev_0 = np.array([20.94889204, 20.96041284, 20.96368521, 20.96826309, 20.95849212])
+        expected_sat_elev_204 = np.array([89.24533884, 89.22663677, 89.25079817, 89.24938043, 89.23004118])
+        sat_azi, sat_elev = self.reader.get_sat_angles()
+        np.testing.assert_allclose(sat_azi[:, 0], expected_sat_azi_0, atol=1.0)
+        np.testing.assert_allclose(sat_azi[:, 2], expected_sat_azi_201, atol=35.0)  # Azi bad close to center!
+        np.testing.assert_allclose(sat_azi[:, -1], expected_sat_azi_408, atol=1.0)
+        np.testing.assert_allclose(sat_elev[:, 0], expected_sat_elev_0, atol=1.0)
+        np.testing.assert_allclose(sat_elev[:, 5], expected_sat_elev_204, atol=1.0)
+
     @mock.patch('pygac.gac_reader.GACReader._get_corrupt_mask')
     def test_get_angles(self, get_corrupt_mask):
         """Test get_angles function of the reader."""
@@ -318,7 +471,6 @@ class TestGacReader(unittest.TestCase):
              315754371969]).astype('datetime64[ms]')
         self.reader.spacecrafts_orbital = {25: 'tiros n'}
         self.reader.spacecraft_id = 25
-        self.reader.times = self.reader.to_datetime(self.reader.utcs)
         expected_sat_azi = np.array(
             [-76.90, 11.08, 145.33, -50.01, np.nan])[:, np.newaxis]
         expected_sun_azi = np.array(
@@ -423,7 +575,6 @@ class TestGacReader(unittest.TestCase):
         self.reader.utcs = np.array([315748035469, 315748359969,
                                      315751135469, 315754371969,
                                      315754371969]).astype('datetime64[ms]')
-        self.reader.times = self.reader.to_datetime(self.reader.utcs)
         corr = self.reader.get_sun_earth_distance_correction()
         numpy.testing.assert_almost_equal(corr, 0.96660494, decimal=7)
 
@@ -451,15 +602,3 @@ class TestGacReader(unittest.TestCase):
                    'gac_header': {'foo': 'bar'},
                    'calib_coeffs_version': 'version'}
         self.assertDictEqual(self.reader.meta_data, mda_exp)
-
-
-def suite():
-    """Test suite for test_reader."""
-    loader = unittest.TestLoader()
-    mysuite = unittest.TestSuite()
-    mysuite.addTest(loader.loadTestsFromTestCase(TestGacReader))
-    return mysuite
-
-
-if __name__ == '__main__':
-    unittest.main()
